//
// = FILENAME
//    
//
// = FUNCTION
//
// = AUTHOR(S)
//    Patric Jensfelt
//
// = COPYRIGHT
//    Copyright (c) 2006 Patric Jensfelt
//
/*----------------------------------------------------------------------*/

#ifndef Mineagent_hh
#define Mineagent_hh

#include <string>

/**
 * This is a but ugly way to wrap all the information that can come
 * from the server
 */
class MineInfo {
public:
  /// Different type of info
  enum Info {
    SQUARE_INFO = 1,
    ILLEGAL_MOVE,
    ALREADY_OPEN,
    PLAYER_STOPPED,
    UNKNOWN_CMD,
    SCORE,
    ALREADY_PEEKED,
  };

  /// The type of info SQUARE_INFO, PLAYER_STOPS, etc
  int m_InfoType;
  
  /// This variable gives the ID of the player that resulted in this
  /// info by opening a square or stopping
  int m_PlayerID;
  
  /// Specifies what square
  int m_X;
  int m_Y;
  
  /// 0-8 for number of mines around,
  /// -1  for mine that just killed you
  /// -2  for mine that occured before the game started and thus does
  ///     not kill you
  /// -10 square outside mine filed
  int m_Value;  
  
  /// The Score in case of a SCORE message
  int m_Score;
  
  /// The command that was unknown
  std::string m_UnknownCmd;
};


/**
 * Class that defines the move that of the agent
 */
class MineMove {
public:
  /// The different type of moves
  enum Move {
    OPEN_SQUARE = 1,
    STOP,
    PEEK_SQUARE
  };

  /// Should be either OPEN_SQUARE or STOP
  int m_Move;
  
  /// Should be the x-coordinate of the square to open
  int m_X;
  
  /// Should be the y-coordinate of the square to open
  int m_Y;
  
  MineMove(int move)
    :m_Move(move)
  {}
};

class MineAgent {
public:
  MineAgent(const std::string name)
    :m_Name(name)
  {}

  virtual ~MineAgent() {}

  /// @return name of the agent
  std::string getName() const { return m_Name; }

  /**
   * @param numPlayers tells how many players are playing this game
   * @param playerID the id for this player in the game
   * @param xsize number of squares in the horizontal direction
   * @param ysize number of squares in the vertical direction 
   * @param deadCost cost to die
   * @param peekCost the cos to take a peek at a square
   * 
   * @return true if initialization went ok
   */
  virtual bool startNewGame(int numPlayers, int playerID,
                            int xsize, int ysize,
                            int deadCost, int peekCost) = 0;
  
  /**
   * @return the move suggested by this agent given the information
   * it has gathered so far
   */
  virtual MineMove getNextMove() = 0;

  /**
   * This function is called by the system after a square is opened,
   * another player has died or stopped playing.
   *
   * @param info information about the game
   */
  virtual void handleInfo(MineInfo info) = 0;

protected:
  /// The name of the agent
  std::string m_Name;
};

#endif // Mineagent_hh
