//
// = FILENAME
//    MineClient.cc
//
// = FUNCTION
//
// = AUTHOR(S)
//    Patric Jensfelt
//
// = COPYRIGHT
//    Copyright (c) 2006 Patric Jensfelt
//
/*----------------------------------------------------------------------*/

#include "MineClient.hh"
#include "StupidAgent.hh"

#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <netdb.h>
#include <iostream>
#include <sstream>

MineClient::MineClient(MineAgent &agent)
  :m_Agent(&agent),
   m_Socket(-1)
{}

void 
MineClient::playGame(const std::string &hostname, int port)
{
  if (!this->connect(hostname, port)) return;

  char cmd[512];

  while (true) {

    std::string wholeMsg = waitForMsg();
    if (wholeMsg == "") break;

    while (wholeMsg != "") {

      if (wholeMsg.length() == 0) break;

      size_t pos = wholeMsg.find_first_of("\n");
      std::string msg = wholeMsg.substr(0,pos);
      wholeMsg = wholeMsg.substr(pos+1,wholeMsg.length()-pos-1);

      if (msg.length() == 0) break;

      //std::cerr << "msg=\"" << msg << "\"\n";

      if (msg.substr(0,4) == "MOVE") {      
        MineMove mm = m_Agent->getNextMove();
        
        if (mm.m_Move == MineMove::OPEN_SQUARE) {
          sprintf(cmd, "OPEN %d %d\n", mm.m_X, mm.m_Y);
        } else if (mm.m_Move == MineMove::PEEK_SQUARE) {
          sprintf(cmd, "PEEK %d %d\n", mm.m_X, mm.m_Y);
        } else if (mm.m_Move == MineMove::STOP) {
          sprintf(cmd, "STOP\n");
        } else {
          std::cerr << "Unexpected command " << mm.m_Move << std::endl;
          continue;
        }
        sendMsg(cmd);
      } else if (msg.substr(0,5) == "START") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk 
            >> args[0] >> args[1] >> args[2]
            >> args[3] >> args[4] >> args[5]) {
          m_Agent->startNewGame(args[0], args[1], args[2],
                                args[3], args[4], args[5]);
        } else {
          std::cerr << "START format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,11) == "SQUARE_INFO") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0] >> args[1] >> args[2] >> args[3]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::SQUARE_INFO;
          mi.m_PlayerID = args[0];
          mi.m_X = args[1];
          mi.m_Y = args[2];
          mi.m_Value = args[3];
          m_Agent->handleInfo(mi);
        } else {
          std::cerr << "SQUARE_INFO format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,12) == "ILLEGAL_MOVE") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0] >> args[1] >> args[2] >> args[3]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::ILLEGAL_MOVE;
          mi.m_PlayerID = args[0];
          mi.m_X = args[1];
          mi.m_Y = args[2];
          mi.m_Value = args[3];
          m_Agent->handleInfo(mi);
        } else {
          std::cerr << "ILLEGAL_MOVE format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,12) == "ALREADY_OPEN") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0] >> args[1] >> args[2]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::ALREADY_OPEN;
          mi.m_PlayerID = args[0];
          mi.m_X = args[1];
          mi.m_Y = args[2];
          m_Agent->handleInfo(mi);
        } else {
          std::cerr << "ALREADY_OPEN format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,14) == "ALREADY_PEEKED") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::ALREADY_PEEKED;
          mi.m_PlayerID = args[0];
          m_Agent->handleInfo(mi);
        } else {
          std::cerr << "ALREADY_PEEKED format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,12) == "PLAYER_STOPS") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::PLAYER_STOPPED;
          mi.m_PlayerID = args[0];
          m_Agent->handleInfo(mi);
        } else {
          std::cerr << "PLAYER_STOPPED format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      } else if (msg.substr(0,11) == "UNKNOWN_CMD") {
        MineInfo mi;
        mi.m_InfoType = MineInfo::UNKNOWN_CMD;
        mi.m_UnknownCmd = msg.substr(12, msg.length());
        m_Agent->handleInfo(mi);
      } else if (msg.substr(0,4) == "DONE") {
        std::istringstream str(msg);
        std::string junk;
        int args[6];
        if (str >> junk >> args[0]) {
          MineInfo mi;
          mi.m_InfoType = MineInfo::SCORE;
          mi.m_Score = args[0];
          m_Agent->handleInfo(mi);

          // We have 

        } else {
          std::cerr << "DONE format is wrong \""
                    << msg << "\"\n";
          continue;
        }
      }
    }
  }
}

bool
MineClient::connect(const std::string &hostname, int port) 
{
  std::cerr << "Trying to connect to " << hostname 
            << " on port " << port << std::endl;

  struct sockaddr_in servername;
  struct hostent *hostinfo;

  /* Create the socket. */
  m_Socket = socket(PF_INET, SOCK_STREAM, 0);
  if (m_Socket < 0) {
    perror ("MineClient::connect socket");
    return false;
  }

  servername.sin_family = AF_INET;
  servername.sin_port = htons (port);
  hostinfo = gethostbyname (hostname.c_str());
  if (hostinfo == NULL) {
    std::cerr << "MineClient::connect Unknown host \""
              << hostname << "\"\n";
    return false;
  }
  servername.sin_addr = *(struct in_addr *) hostinfo->h_addr;

  /* Connect to the server. */
  if (::connect(m_Socket, (struct sockaddr *) &servername, 
                  sizeof (servername)) < 0) {
    perror("MineClient::connect connect\n");
    return false;
  }

  std::string msg = waitForMsg();
  if (msg == "" ||
      msg.substr(0,31) != "WELCOME to the Minesweep server") {
    std::cerr << "Did not get greeting message\n";
    return false;
  } 

  std::cerr << "Got msg \"" << msg << "\"\n";

  sendMsg(m_Agent->getName() + "\n");

  return true;
}

std::string
MineClient::waitForMsg()
{
  fd_set mask;
  FD_ZERO(&mask);
  FD_SET(m_Socket, &mask);

  int selret = select(FD_SETSIZE, &mask, NULL, NULL, NULL);    
  if (selret > 0) {
    // We got data
    char buf[512];
    int n = read(m_Socket, buf, 512);
    buf[n] = 0;
    return buf;
  } else {
    return "";
  }    
}

bool
MineClient::sendMsg(const std::string &msg)
{    
  if (write(m_Socket, msg.c_str(), msg.length()) !=
      (int)msg.length()) {
    std::cerr << "MineClient, could not send whole message\n";
    return false;
  }

  return true;
}
  
bool
MineClient::disconnect() 
{
  if (m_Socket == -1) return true;

  std::cerr << "Closing connection to server" << std::endl;

  close(m_Socket);
  m_Socket = -1;
  return true;
}

int main(int argc, char **argv) 
{
  StupidAgent sa;
  MineClient mc(sa);
  mc.playGame("localhost", 5555);
}
