//
// = FILENAME
//    StupidAgent.cc
//
// = FUNCTION
//
// = AUTHOR(S)
//    Patric Jensfelt
//
// = COPYRIGHT
//    Copyright (c) 2006 Patric Jensfelt
//
/*----------------------------------------------------------------------*/

#include "StupidAgent.hh"

#include <iostream>

StupidAgent::StupidAgent()
  :MineAgent("StupidAgent"),
   m_Opened(0),
   m_XSize(0),
   m_YSize(0),
   m_TryPeeking(true)
{}

StupidAgent::~StupidAgent()
{
  freeMemory();
}

void 
StupidAgent::freeMemory()
{
  if (m_Opened) {
    delete [] m_Opened;
  }
  m_Opened = 0;
}

bool 
StupidAgent::startNewGame(int numPlayers, int playerID,
                          int xsize, int ysize,
                          int deadCost, int peekCost)
{
  freeMemory();

  std::cerr << "New game with xsize=" << xsize
            << " ysize=" << ysize 
            << " deadCost=" << deadCost
            << " peekCost=" << peekCost
            << std::endl;

  m_Opened = new char[m_XSize*m_YSize];
  for (int i = 0; i < m_XSize; i++) {
    for (int j = 0; j < m_YSize; j++) {
      m_Opened[i + j * m_XSize] = false;
    }
  }

  m_XSize = xsize;
  m_YSize = ysize;

  return true;
}

MineMove 
StupidAgent::getNextMove()
{
  std::cerr << "getNextMove\n";

  MineMove mm(MineMove::OPEN_SQUARE);

  if (m_TryPeeking) {
    mm.m_Move = MineMove::PEEK_SQUARE;
    mm.m_X = 0;
    mm.m_Y = 0;
    std::cerr << "Peeking\n";
    return mm;
  }

  for (int i = 0; i < m_XSize; i++) {
    for (int j = 0; j < m_YSize; j++) {
      if (m_Opened[i + j * m_XSize] == false) {
        mm.m_X = i;
        mm.m_Y = j;
        m_Opened[i + j * m_XSize] = true;
        return mm;
      }
    }
  }
        
  mm.m_Move = MineMove::STOP;

  return mm;
}

void 
StupidAgent::handleInfo(MineInfo info)
{
  std::cerr << "StupidAgent::handleInfo " << info.m_InfoType << std::endl;
  if (info.m_InfoType == MineInfo::ALREADY_PEEKED) {
    m_TryPeeking = false;
    std::cerr << "Not allowed to peek anymore\n";
  } else if (info.m_InfoType == MineInfo::SQUARE_INFO) {
    std::cerr << "Square info : " << info.m_Value << std::endl;
  }
}
